//! \file    Translator.hpp
//! \brief   Declaration of the Translator class, which handles both project
//!          description analysis and code generation.
//! \version 2007/10/01 (yyyy/mm/dd)
//! \author  Luigi Nardi <luiginardi(at)gmail.com>
//
/*
Copyright or © or Copr. Luigi NARDI

mail of the author: luiginardi@gmail.com

This software is a framework for variational data assimilation in
numerical models.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/



#ifndef YAO_TRANSLATOR_HPP_INCLUDED
#define YAO_TRANSLATOR_HPP_INCLUDED

#include <limits>
#include <cstring>
#include "analyzers/BaseParser.hpp"
#include "Tokenizer.hpp"
//#include "TemplateAnalyzer.hpp"

// SOME DEFINE /////////////////////////////////////////////////////////////
#define VERSION "v9.2"
#define SYSTEM "linux"
#define REVISION "$Rev$"

#define YSTATE	"Ystate"
#define YGRAD	"Ygrad"
#define YWISH	"Ywish"
#define YEPSI	"Yepsi"
#define YI	"Yi"
#define YJ	"Yj"
#define YK	"Yk"
#define YTEMPS	"YTemps"
#define YDELTA  "Ydelta"


/* MACROS //////////////////////////////////////////////// Y:NOWARD: U:!NOWARD T:TEMPO //Y:OUTER U:INNER
======================================= without OPTION ======================================*/
// Generic access macros (for Ydelta, Ystate, Ygrad)
// Access to any modul's output (output, time, coordinates in space)
#define MACRO1_VALST(Modul, C, sg)	sprintf(buffer, "/*M1_VALST  */ #define \t %s_%s(v, i, t) \t\t Y%s->%s[t][i][v] \n/*M1_VALST  */ #define \t Y%c_%s(v, i, t) \t\t Y%s->%s[t][i][v] \n", sg, Modul, Modul, sg, C, Modul, Modul, sg)
#define MACRO2_VALST(Modul, C, sg)	sprintf(buffer, "/*M2_VALST  */ #define \t %s_%s(v, i, j, t) \t\t Y%s->%s[t][i][j][v] \n/*M2_VALST  */ #define \t Y%c_%s(v, i, j, t) \t\t Y%s->%s[t][i][j][v] \n", sg, Modul, Modul, sg, C, Modul, Modul, sg)
#define MACRO3_VALST(Modul, C, sg)	sprintf(buffer, "/*M3_VALST  */ #define \t %s_%s(v, i, j, k, t) \t Y%s->%s[t][i][j][k][v] \n/*M3_VALST  */ #define \t Y%c_%s(v, i, j, k, t) \t\t Y%s->%s[t][i][j][k][v] \n", sg, Modul, Modul, sg, C, Modul, Modul, sg)

// Access to named modul's output (time, coordinates in space)
#define MACRO1_VALSIT(Modul, C, sg, wn)	sprintf(buffer, "/*M1_VALSIT */ #define \t Y%c%d_%s(i, t) \t\t Y%s->%s[t][i][%d] \n", C, wn+1, Modul, Modul, sg, wn)
#define MACRO2_VALSIT(Modul, C, sg, wn)	sprintf(buffer, "/*M2_VALSIT */ #define \t Y%c%d_%s(i, j, t) \t\t Y%s->%s[t][i][j][%d] \n", C, wn+1, Modul, Modul, sg, wn)
#define MACRO3_VALSIT(Modul, C, sg, wn)	sprintf(buffer, "/*M3_VALSIT */ #define \t Y%c%d_%s(i, j, k, t) \t\t Y%s->%s[t][i][j][k][%d] \n", C, wn+1, Modul, Modul, sg, wn)

// Access to named modul's output (coordinate in space, current time)
#define MACRO1_HERESIT(Modul, C, sg, wn)  sprintf(buffer, "/*M1_HERESIT*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%s][%d] \n", C, wn+1, Modul, Modul, sg, YTEMPS, YI, wn)
#define MACRO2_HERESIT(Modul, C, sg, wn)  sprintf(buffer, "/*M2_HERESIT*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%s][%s][%d] \n", C, wn+1, Modul, Modul, sg, YTEMPS, YI, YJ, wn)
#define MACRO3_HERESIT(Modul, C, sg, wn)  sprintf(buffer, "/*M3_HERESIT*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%s][%s][%s][%d] \n", C, wn+1, Modul, Modul, sg, YTEMPS, YI, YJ, YK, wn)

// Number of moduls, inputs and outputs.
#define MACRO0_NBMOD(Modul, Wnb) 	 sprintf(buffer, "/*NBMOD     */ #define \t YNB_%s \t %d\n", Modul, Wnb)
#define MACRO0_NBSMOD(Modul, Wnbs) sprintf(buffer, "/*NBSMOD    */ #define \t YNBS_%s \t %d\n", Modul, Wnbs)
#define MACRO0_NBIMOD(Modul, Wnbi) sprintf(buffer, "/*NBIMOD    */ #define \t YNBI_%s \t %d\n", Modul, Wnbi)

// Code simplification
// No need of this.. (don't exist )
//#define MACRO1_IRMOD(Modul, a)		sprintf(buffer, "/*M1_IRMOD  */ #define \t Y%s(%c) \t Y%s[%c]\n", Modul, a, Modul, a)
//#define MACRO2_IRMOD(Modul, a, b) sprintf(buffer, "/*M2_IRMOD  */ #define \t Y%s(%c, %c) \t Y%s[%c][%c]\n", Modul, a, b, Modul, a, b)
//#define MACRO3_IRMOD(Modul) 			sprintf(buffer, "/*M3_IRMOD  */ #define \t Y%s(i, j, k) \t Y%s[i][j][k]\n", Modul, Modul)

//#define MACRO1_NOWMOD(Modul, a) 		sprintf(buffer, "/*M1_NOWMOD */ #define \t Y%sY \t\t Y%s[Y%c]\n", Modul, Modul, a)
//#define MACRO2_NOWMOD(Modul, a, b) 	sprintf(buffer, "/*M2_NOWMOD */ #define \t Y%sY \t\t Y%s[Y%c][Y%c]\n", Modul, Modul, a, b)
//#define MACRO3_NOWMOD(Modul) 				sprintf(buffer, "/*M3_NOWMOD */ #define \t Y%sY \t\t Y%s[%s][%s][%s]\n", Modul, Modul, YI, YJ, YK)

// Yadjust : fonctions d'ajustement standard : state -= epsi * grad.
#define MACRO1_ADJUS(Modul, Space) sprintf(buffer, "/*M1_ADJUS*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1) -= Yepsi_%s(Yws, Yw1) * Ygrad_%s(Yws, Yw1); \n\t}\n}", Modul, Space, Modul, Modul, Modul, Modul)
#define MACRO2_ADJUS(Modul, Space) sprintf(buffer, "/*M2_ADJUS*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1, Yw2) -= Yepsi_%s(Yws, Yw1, Yw2) * Ygrad_%s(Yws, Yw1, Yw2);\n\t}\n}", Modul, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO3_ADJUS(Modul, Space) sprintf(buffer, "/*M3_ADJUS*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1, Yw2, Yw3) -= Yepsi_%s(Yws, Yw1, Yw2, Yw3) * Ygrad_%s(Yws, Yw1, Yw2, Yw3);\n\t}\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO1_ADJUST(Modul, Space) sprintf(buffer, "/*M1_ADJUST*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Ywt) -= Yepsi_%s(Yws, Yw1) * Ygrad_%s(Yws, Yw1, Ywt); \n\t\t}\n}", Modul, Space, Modul, Modul, Modul, Modul, Modul, Modul)
#define MACRO2_ADJUST(Modul, Space) sprintf(buffer, "/*M2_ADJUST*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Yw2, Ywt) -= Yepsi_%s(Yws, Yw1, Yw2) * Ygrad_%s(Yws, Yw1, Yw2, Ywt);\n\t}\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, Modul, Modul)
#define MACRO3_ADJUST(Modul, Space) sprintf(buffer, "/*M3_ADJUST*/ \nvoid Yadjust_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Yw2, Yw3, Ywt) -= Yepsi_%s(Yws, Yw1, Yw2, Yw3) * Ygrad_%s(Yws, Yw1, Yw2, Yw3, Ywt);\n\t}\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, Modul, Modul)

// gradtest

/*YgradCTOtab : fonctions de cumul des gradients dans un tableau (pour basic intern loop) */
#define MACRO1_GCTOTB(Modul, Space)  sprintf(buffer, "/*M1_GCTOTB  */ \nvoid YgradCTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1); ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GCTOTB(Modul, Space)  sprintf(buffer, "/*M2_GCTOTB  */ \nvoid YgradCTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1, Yw2); ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GCTOTB(Modul, Space)  sprintf(buffer, "/*M3_GCTOTB  */ \nvoid YgradCTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1, Yw2, Yw3); ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_GCTOTBT(Modul, Space) sprintf(buffer, "/*M1_GCTOTBT */ \nvoid YgradCTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1, Ywt); ++Y3windice;}}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GCTOTBT(Modul, Space) sprintf(buffer, "/*M2_GCTOTBT */ \nvoid YgradCTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1, Yw2, Ywt); ++Y3windice;}}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GCTOTBT(Modul, Space) sprintf(buffer, "/*M3_GCTOTBT */ \nvoid YgradCTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t tab[Y3windice] += YG_%s(Yws, Yw1, Yw2, Yw3, Ywt); ++Y3windice;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul)

// Definition of the class.
#define MACRO0_PCL(Modul,Parm)	sprintf(buffer, "\n/*M_PCL   */\nclass \t %s:public Yao%s \n{	public: \n\t %s(){}; \n\t ~%s (){}; \n\t void forward (%s); \n\t void backward(%s);  \n};\n", Modul, Modul, Modul, Modul, Parm, Parm)
#define MACRO0_PCLNW(Modul,Parm)	sprintf(buffer, "\n/*M_PCLNW */\nclass \t %s:public Yao%s \n{	public: \n\t %s(){}; \n\t ~%s (){}; \n\t void forward (%s); \n\t void backward(%s);  \n\t void flinward (%s); \n};\n", Modul, Modul, Modul, Modul, Parm, Parm, Parm)
#define MACRO0_PCLAN(Modul,Autonet)	sprintf(buffer, "\n/*M_PCLAN */\nclass \t %s:public Yao%s \n{	public: \n\t %s(){}; \n\t ~%s (){}; \n\t void forward (){Ynet_fward2(YNID_%s, %s);} \n\t void backward(){Ynet_bward2(YNID_%s, %s);}  \n\t void	flinward (){Ynet_flward2(YNID_%s, %s, %s);} \n};\n", Modul, Modul, Modul, Modul, Autonet, YSTATE, Autonet, YGRAD, Autonet, YSTATE, YGRAD)
#define MACRO0_PCLANT(Modul,Autonet) sprintf(buffer, "\n/*M_PCLANT*/\nclass \t %s:public Yao%s \n{	public: \n\t %s(){}; \n\t ~%s (){}; \n\t void forward (){Ynet_fward2(YNID_%s, %s[%s]);} \n\t void backward(){Ynet_bward2(YNID_%s, %s[%s]);}  \n\t void	flinward (){Ynet_flward2(YNID_%s, %s[%s], %s[%s]);} \n};\n", Modul, Modul, Modul, Modul, Autonet, YSTATE, YTEMPS, Autonet, YGRAD, YTEMPS, Autonet, YSTATE, YTEMPS, YGRAD, YTEMPS)


// Size of the axe.
#define MACRO_AXEY(a, Modul, Wa1)		sprintf(buffer, "/*AXEY      */ #define \t YA%c_%s \t %d \n", a, Modul, Wa1)

// Ysetepsi : fonctions valorization of the epsilons.
#define MACRO1_SEPS(Modul, Space) sprintf(buffer, "/*M1_SEPS*/ \nvoid Ysetepsi_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y%s->%s[Yw1][Yws] = val;}}\n}", Modul, YREAL, Space, Modul, Modul, YEPSI)
#define MACRO2_SEPS(Modul, Space) sprintf(buffer, "/*M2_SEPS*/ \nvoid Ysetepsi_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Yw1][Yw2][Yws] = val;}}}\n}", Modul, YREAL, Space, Space, Modul, Modul, YEPSI)
#define MACRO3_SEPS(Modul, Space) sprintf(buffer, "/*M3_SEPS*/ \nvoid Ysetepsi_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Yw1][Yw2][Yw3][Yws] = val;}}}}\n}", Modul, YREAL, Space, Space, Space, Modul, Modul, YEPSI)


// Macro de simplification d'ecriture du code.

#define MACRO0_PTRMOD(Cmodul, Modul) sprintf(buffer, "%s \t\t *Y%s;\n", Cmodul, Modul)
#define MACRO1_PTRMOD(Cmodul, Modul, Axa)		sprintf(buffer, "%s \t\t *Y%s[%s];\n", Cmodul, Modul, Axa)
#define MACRO2_PTRMOD(Cmodul, Modul, Axa, Axb)	sprintf(buffer, "%s \t\t *Y%s[%s][%s];\n", Cmodul, Modul, Axa, Axb)
#define MACRO3_PTRMOD(Cmodul, Modul, Axa, Axb, Axc)		sprintf(buffer, "%s \t\t *Y%s[%s][%s][%s];\n", Cmodul, Modul, Axa, Axb, Axc)

#define MACRO1_VALSG(Modul, C, sgwe)	sprintf(buffer, "/*M1_VALSG  */ #define \t %s_%s(v, i) \t\t Y%s->%s[i][v] \n/*M1_VALSG  */ #define \t Y%c_%s(v, i) \t\t Y%s->%s[i][v] \n", sgwe, Modul, Modul, sgwe, C, Modul, Modul, sgwe)
#define MACRO2_VALSG(Modul, C, sgwe)	sprintf(buffer, "/*M2_VALSG  */ #define \t %s_%s(v, i, j) \t\t Y%s->%s[i][j][v] \n/*M2_VALSG  */ #define \t Y%c_%s(v, i, j) \t\t Y%s->%s[i][j][v] \n", sgwe, Modul, Modul, sgwe, C, Modul, Modul, sgwe)
#define MACRO3_VALSG(Modul, C, sgwe)	sprintf(buffer, "/*M3_VALSG  */ #define \t %s_%s(v, i, j, k) \t Y%s->%s[i][j][k][v] \n/*M3_VALSG  */ #define \t Y%c_%s(v, i, j, k) \t\t Y%s->%s[i][j][k][v] \n", sgwe, Modul, Modul, sgwe, C, Modul, Modul, sgwe)

#define MACRO1_VALSGI(Modul, C, sgwe, wn)	sprintf(buffer, "/*M1_VALSGI */ #define \t Y%c%d_%s(i) \t\t Y%s->%s[i][%d] \n", C, wn+1, Modul, Modul, sgwe, wn)
#define MACRO2_VALSGI(Modul, C, sgwe, wn)	sprintf(buffer, "/*M2_VALSGI */ #define \t Y%c%d_%s(i, j) \t\t Y%s->%s[i][j][%d] \n", C, wn+1, Modul, Modul, sgwe, wn)
#define MACRO3_VALSGI(Modul, C, sgwe, wn)	sprintf(buffer, "/*M3_VALSGI */ #define \t Y%c%d_%s(i, j, k) \t\t Y%s->%s[i][j][k][%d] \n", C, wn+1, Modul, Modul, sgwe, wn)

#define MACRO1_HERESGI(Modul, C, sgw, wn)	sprintf(buffer, "/*M1_HERESGI*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%d] \n", C, wn+1, Modul, Modul, sgw, YI, wn)
#define MACRO2_HERESGI(Modul, C, sgw, wn)	sprintf(buffer, "/*M2_HERESGI*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%s][%d] \n", C, wn+1, Modul, Modul, sgw, YI, YJ, wn)
#define MACRO3_HERESGI(Modul, C, sgw, wn)	sprintf(buffer, "/*M3_HERESGI*/ #define \t Y%c%dY_%s \t\t Y%s->%s[%s][%s][%s][%d] \n", C, wn+1, Modul, Modul, sgw, YI, YJ, YK, wn)



//#define MACRO1_CREY(Modul, Space) sprintf(buffer, "/*M1_CREY*/ \nvoid Ycreate_%s() 	\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t if ((Y%s(Yw1) = new Yao%s ())==NULL) Ycreate_errnew();}\n}", Modul, Space, Modul, Modul)
//#define MACRO2_CREY(Modul, Space) sprintf(buffer, "/*M2_CREY*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t if ((Y%s(Yw1, Yw2)= new Yao%s ())==NULL) Ycreate_errnew();}}\n}", Modul, Space, Space, Modul, Modul)
//#define MACRO3_CREY(Modul, Space) sprintf(buffer, "/*M3_CREY*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t if ((Y%s(Yw1, Yw2, Yw3) = new Yao%s ())==NULL) Ycreate_errnew();}}}\n}", Modul, Space, Space, Space, Modul, Modul)
//#define MACRO1_CREU(Modul, Space) sprintf(buffer, "/*M1_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t if ((Y%s(Yw1) = new %s ())==NULL) Ycreate_errnew();}\n}", Modul, Space, Modul, Modul)
//#define MACRO2_CREU(Modul, Space) sprintf(buffer, "/*M2_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t if ((Y%s(Yw1, Yw2)= new %s ())==NULL) Ycreate_errnew();}}\n}", Modul, Space, Space, Modul, Modul)
//#define MACRO3_CREU(Modul, Space) sprintf(buffer, "/*M3_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t if ((Y%s(Yw1, Yw2, Yw3) = new %s ())==NULL) Ycreate_errnew();}}}\n}", Modul, Space, Space, Space, Modul, Modul)

#define MACRO0_CREY(Modul) sprintf(buffer, "/*M0_CREY*/ \nvoid Ycreate_%s() 	\n{\n\t if ((Y%s = new Yao%s ())==NULL) \n\t\t Ycreate_errnew();\n}", Modul,Modul,Modul)
#define MACRO0_CREU(Modul) sprintf(buffer, "/*M0_CREU*/ \nvoid Ycreate_%s()  \n{\n\t if ((Y%s = new %s ())==NULL) \n\t\t Ycreate_errnew();\n}", Modul,Modul,Modul)

//#define MACRO2_CREY(Modul, Space) sprintf(buffer, "/*M2_CREY*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t if ((Y%s(Yw1, Yw2)= new Yao%s ())==NULL) Ycreate_errnew();}}\n}", Modul, Space, Space, Modul, Modul)
//#define MACRO3_CREY(Modul, Space) sprintf(buffer, "/*M3_CREY*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t if ((Y%s(Yw1, Yw2, Yw3) = new Yao%s ())==NULL) Ycreate_errnew();}}}\n}", Modul, Space, Space, Space, Modul, Modul)
//#define MACRO1_CREU(Modul, Space) sprintf(buffer, "/*M1_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t if ((Y%s(Yw1) = new %s ())==NULL) Ycreate_errnew();}\n}", Modul, Space, Modul, Modul)
//#define MACRO2_CREU(Modul, Space) sprintf(buffer, "/*M2_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t if ((Y%s(Yw1, Yw2)= new %s ())==NULL) Ycreate_errnew();}}\n}", Modul, Space, Space, Modul, Modul)
//#define MACRO3_CREU(Modul, Space) sprintf(buffer, "/*M3_CREU*/ \nvoid Ycreate_%s()  \n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t if ((Y%s(Yw1, Yw2, Yw3) = new %s ())==NULL) Ycreate_errnew();}}}\n}", Modul, Space, Space, Space, Modul, Modul)


// Yrazgrad : fonctions de remise a zero des gradients.
#define MACRO1_RGRAD(Modul, Space) sprintf(buffer, "/*M1_RGRAD*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y%s->%s[Yw1][Yws] = 0.0;}}\n}", Modul, Space, Modul, Modul, YGRAD)
#define MACRO2_RGRAD(Modul, Space) sprintf(buffer, "/*M2_RGRAD*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Yw1][Yw2][Yws] = 0.0;}}}\n}", Modul, Space, Space, Modul, Modul, YGRAD)
#define MACRO3_RGRAD(Modul, Space) sprintf(buffer, "/*M3_RGRAD*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Yw1][Yw2][Yw3][Yws] = 0.0;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YGRAD)
#define MACRO1_RGRADT(Modul, Space, NbPas) sprintf(buffer, "/*M1_RGRADT*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=0; Ywt<%s; ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Ywt][Yw1][Yws] = 0.0;}}}\n}", Modul, Space, NbPas, Modul, Modul, YGRAD)
#define MACRO2_RGRADT(Modul, Space, NbPas) sprintf(buffer, "/*M2_RGRADT*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=0; Ywt<%s; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Ywt][Yw1][Yw2][Yws] = 0.0;}}}}\n}", Modul, Space, Space, NbPas, Modul, Modul, YGRAD)
#define MACRO3_RGRADT(Modul, Space, NbPas) sprintf(buffer, "/*M3_RGRADT*/ \nvoid Yrazgrad_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=0; Ywt<%s; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws] = 0.0;}}}}}\n}", Modul, Space, Space, Space, NbPas, Modul, Modul, YGRAD)

// Youtoobs : pour recuperer les etats de sortie comme etats desires (observations)sert de la meme facon pour les termes d'ebauche!
#define MACRO0_OUTOB(Modul)   sprintf(buffer, "/*M0_OUTOB*/ \nint Youtoobs_%s(YioKind yiokind, int Yws, int Ywt, int arbpdt)\n{\n	 Yaobs.time = arbpdt;\n	 Yaobs.imod = Yimod(\"%s\");\n	 Yaobs.smod = Yws;", Modul, Modul)
#define MACRO1_OUTOB(Modul, Space)  sprintf(buffer, "/*M1_OUTOB*/ \n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, -1, -1, Ywt, &Ystate_%s(Yws, Yw1)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = -1; Yaobs.kaxe = -1;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Modul, Modul, Modul)
#define MACRO2_OUTOB(Modul, Space)  sprintf(buffer, "/*M2_OUTOB*/ \n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, Yw2, -1, Ywt, &Ystate_%s(Yws, Yw1, Yw2)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = Yw2; Yaobs.kaxe = -1;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1, Yw2);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Space, Modul, Modul, Modul)
#define MACRO3_OUTOB(Modul, Space)  sprintf(buffer, "/*M3_OUTOB*/ \n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &Ystate_%s(Yws, Yw1, Yw2, Yw3)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = Yw2; Yaobs.kaxe = Yw3;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1, Yw2, Yw3);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_OUTOBT(Modul, Space) sprintf(buffer, "/*M1_OUTOBT*/\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, -1, -1, Ywt, &Ystate_%s(Yws, Yw1, Ywt)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = -1; Yaobs.kaxe = -1;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1, Ywt);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Modul, Modul, Modul)
#define MACRO2_OUTOBT(Modul, Space) sprintf(buffer, "/*M2_OUTOBT*/\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, Yw2, -1, Ywt, &Ystate_%s(Yws, Yw1, Yw2, Ywt)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = Yw2; Yaobs.kaxe = -1;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1, Yw2, Ywt);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Space, Modul, Modul, Modul)
#define MACRO3_OUTOBT(Modul, Space) sprintf(buffer, "/*M3_OUTOBT*/\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t {\tif(select_io(yiokind, \"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &Ystate_%s(Yws, Yw1, Yw2, Yw3, Ywt)))\n\t\t\t{\tYaobs.iaxe = Yw1; Yaobs.jaxe = Yw2; Yaobs.kaxe = Yw3;\n\t\t\t\tYaobs.vobs = Ystate_%s(Yws, Yw1, Yw2, Yw3, Ywt);\n\t\t\t\tif(!Yobs_insert (&Yaobs)) return(0);\n\t\t\t}\n\t }", Space, Space, Space, Modul, Modul, Modul)
#define MACRO9_OUTOB()        sprintf(buffer, "/*M9_OUTOB*/  \n	 return(1);\n}")

// Ysetstate : fonctions valorisation des etats.
// done
#define MACRO1_SETA(Modul, Space) sprintf(buffer, "/*M1_SETA*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y%s->%s[Yw1][Yws] = val;}}\n}", Modul, YREAL, Space, Modul, Modul, YSTATE)
#define MACRO2_SETA(Modul, Space) sprintf(buffer, "/*M2_SETA*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Yw1][Yw2][Yws] = val;}}}\n}", Modul, YREAL, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_SETA(Modul, Space) sprintf(buffer, "/*M3_SETA*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Yw1][Yw2][Yw3][Yws] = val;}}}}\n}", Modul, YREAL, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_SETAT(Modul, Space, NbPas) sprintf(buffer, "/*M1_SETAT*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=0; Ywt<%s;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Ywt][Yw1][Yws] = val;}}}\n}", Modul, YREAL, Space, NbPas, Modul, Modul, YSTATE)
#define MACRO2_SETAT(Modul, Space, NbPas) sprintf(buffer, "/*M2_SETAT*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=0; Ywt<%s;    ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Ywt][Yw1][Yw2][Yws] = val;}}}}\n}", Modul, YREAL, Space, Space, NbPas, Modul, Modul, YSTATE)
#define MACRO3_SETAT(Modul, Space, NbPas) sprintf(buffer, "/*M3_SETAT*/ \nvoid Ysetstate_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=0; Ywt<%s; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws] = val;}}}}}\n}", Modul, YREAL, Space, Space, Space, NbPas, Modul, Modul, YSTATE)

// Yio : fonctions d'i/o.
#define MACRO1_YIOTU(Modul) sprintf(buffer, "/*M1_YIOTU*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1), val);\n}", Modul, YREAL, Modul, Modul)
#define MACRO2_YIOTU(Modul) sprintf(buffer, "/*M2_YIOTU*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1, Yw2), val);\n}", Modul, YREAL, Modul, Modul)
#define MACRO3_YIOTU(Modul) sprintf(buffer, "/*M3_YIOTU*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1, Yw2, Yw3), val);\n}", Modul, YREAL, Modul, Modul)
#define MACRO1_YIOUT(Modul) sprintf(buffer, "/*M1_YIOUT*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1, Ywt), val);\n}", Modul, YREAL, Modul, Modul)
#define MACRO2_YIOUT(Modul) sprintf(buffer, "/*M2_YIOUT*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1, Yw2, Ywt), val);\n}", Modul, YREAL, Modul, Modul)
#define MACRO3_YIOUT(Modul) sprintf(buffer, "/*M3_YIOUT*/ \nvoid Yio_%s(int Yws, int Yw1, int Yw2, int Yw3, int Ywt, %s val)	\n{  Yiorwf (\"%s\", Yws, Yw1, Yw2, Yw3, Ywt, &YS_%s(Yws, Yw1, Yw2, Yw3, Ywt), val);\n}", Modul, YREAL, Modul, Modul)

// Ysetwish : fonction de valorisation de la zone Ywish.
#define MACRO1_SWISH(Modul, Space) sprintf(buffer, "/*M1_SWISH*/ \nvoid Ysetwish_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Ywish_%s(Yws, Yw1) = val;}}\n}", Modul, YREAL, Space, Modul, Modul)
#define MACRO2_SWISH(Modul, Space) sprintf(buffer, "/*M2_SWISH*/ \nvoid Ysetwish_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Ywish_%s(Yws, Yw1, Yw2) = val;}}}\n}", Modul, YREAL, Space, Space, Modul, Modul)
#define MACRO3_SWISH(Modul, Space) sprintf(buffer, "/*M3_SWISH*/ \nvoid Ysetwish_%s(%s val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Ywish_%s(Yws, Yw1, Yw2, Yw3) = val;}}}}\n}", Modul, YREAL, Space, Space, Space, Modul, Modul)

// YtabTOgrad : fonctions de transfert d'un tableau vers les gradients.
#define MACRO1_TBTOG(Modul, Space)  sprintf(buffer, "/*M1_TBTOG   */ \nvoid YtabTOgrad_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YG_%s(Yws, Yw1) = tab[Y3windice]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_TBTOG(Modul, Space)  sprintf(buffer, "/*M2_TBTOG   */ \nvoid YtabTOgrad_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Yw2) = tab[Y3windice]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_TBTOG(Modul, Space)  sprintf(buffer, "/*M3_TBTOG   */ \nvoid YtabTOgrad_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3) = tab[Y3windice]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_TBTOGT(Modul, Space) sprintf(buffer, "/*M1_TBTOGT  */ \nvoid YtabTOgrad_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Ywt) = tab[Y3windice]; ++Y3windice;}}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_TBTOGT(Modul, Space) sprintf(buffer, "/*M2_TBTOGT  */ \nvoid YtabTOgrad_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Ywt) = tab[Y3windice]; ++Y3windice;}}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_TBTOGT(Modul, Space) sprintf(buffer, "/*M3_TBTOGT  */ \nvoid YtabTOgrad_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) = tab[Y3windice]; ++Y3windice;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul)

/*======================================== YO_NETWARD ======================================*/
/*Ynetload : fonctions de chargement des poids d'un reseau de neurones */
#ifdef YO_NETWARD
#define MACRO0_NETLOD(NetName, iNet)   sprintf(buffer, "/*M0_NETLOD*/ \nvoid Ynetload_%s(int argc, char *argv[])\n{\t YNBW_%s = Ynet_load(%i, argv[2]);\n\t YNTW_%s = YTabNet[%i].tweight;\n}", NetName, NetName, iNet, NetName, iNet)
#endif
/*-------------------------------------- FIN YO_NETWARD ------------------------------------*/

/*======================================== YO_VARINCR ======================================*/
#ifdef YO_VARINCR
/* Yc_adjustd : fonction d'ajustement pour l'incremental : correction de dx : delta -= epsi * grad */
#define MACRO1_ADJUD(Modul, Space) sprintf(buffer, "/*M1_ADJUD*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t { Ydelta_%s(Yws, Yw1, 0) -= Yepsi_%s(Yws, Yw1) * Ygrad_%s(Yws, Yw1); \n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul)
#define MACRO2_ADJUD(Modul, Space) sprintf(buffer, "/*M2_ADJUD*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ydelta_%s(Yws, Yw1, Yw2, 0) -= Yepsi_%s(Yws, Yw1, Yw2) * Ygrad_%s(Yws, Yw1, Yw2);\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO3_ADJUD(Modul, Space) sprintf(buffer, "/*M3_ADJUD*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ydelta_%s(Yws, Yw1, Yw2, Yw3, 0) -= Yepsi_%s(Yws, Yw1, Yw2, Yw3) * Ygrad_%s(Yws, Yw1, Yw2, Yw3);\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO1_ADJUDT(Modul, Space) sprintf(buffer, "/*M1_ADJUDT*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ydelta_%s(Yws, Yw1, Ywt) -= Yepsi_%s(Yws, Yw1) * Ygrad_%s(Yws, Yw1, Ywt); \n\t\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, Modul, Modul)
#define MACRO2_ADJUDT(Modul, Space) sprintf(buffer, "/*M2_ADJUDT*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ydelta_%s(Yws, Yw1, Yw2, Ywt) -= Yepsi_%s(Yws, Yw1, Yw2) * Ygrad_%s(Yws, Yw1, Yw2, Ywt);\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, Modul, Modul)
#define MACRO3_ADJUDT(Modul, Space) sprintf(buffer, "/*M3_ADJUDT*/ \nvoid Yc_adjustd_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ydelta_%s(Yws, Yw1, Yw2, Yw3, Ywt) -= Yepsi_%s(Yws, Yw1, Yw2, Yw3) * Ygrad_%s(Yws, Yw1, Yw2, Yw3, Ywt);\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, Modul, Modul)

/* Yc_adjustk : fonction d'ajustement pour l'incremental : correction de l'etat : state += delta   i.e; state(k+1) = state(k) + delta */
#define MACRO1_ADJUK(Modul, Space) sprintf(buffer, "/*M1_ADJUK*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1) += Ydelta_%s(Yws, Yw1, 0); \n\t }\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_ADJUK(Modul, Space) sprintf(buffer, "/*M2_ADJUK*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1, Yw2) += Ydelta_%s(Yws, Yw1, Yw2, 0);\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_ADJUK(Modul, Space) sprintf(buffer, "/*M3_ADJUK*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t { Ystate_%s(Yws, Yw1, Yw2, Yw3) += Ydelta_%s(Yws, Yw1, Yw2, Yw3, 0);\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_ADJUKT(Modul, Space) sprintf(buffer, "/*M1_ADJUKT*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yws=0;Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Ywt) += Ydelta_%s(Yws, Yw1, Ywt); \n\t\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, Modul)
#define MACRO2_ADJUKT(Modul, Space) sprintf(buffer, "/*M2_ADJUKT*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Yw2, Ywt) += Ydelta_%s(Yws, Yw1, Yw2, Ywt);\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, Modul)
#define MACRO3_ADJUKT(Modul, Space) sprintf(buffer, "/*M3_ADJUKT*/ \nvoid Yc_adjustk_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1) \n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2) \n\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3) \n\t for (int Yws=0; Yws<YNBS_%s; ++Yws) \n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt) \n\t { Ystate_%s(Yws, Yw1, Yw2, Yw3, Ywt) += Ydelta_%s(Yws, Yw1, Yw2, Yw3, Ywt);\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, Modul)

/*Ysetpdelta : fonctions d'affectation de delta = state * pdx pour les modules target (cf->v7) */
/*Ydeltograd : fonctions d'affectation de grad = delta pour les modules target (cf->v7) */

/*YgradEQPdelta : fonctions d'affectation de grad = delta * pfact */
#define MACRO1_GEQPD(Modul, Space)  sprintf(buffer, "/*M1_GEQPD */ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YG_%s(Yws, Yw1) = YD_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPD(Modul, Space)  sprintf(buffer, "/*M2_GEQPD */ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Yw2) = YD_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPD(Modul, Space)  sprintf(buffer, "/*M3_GEQPD */ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_GEQPDT(Modul, Space) sprintf(buffer, "/*M1_GEQPDT*/ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Ywt) = YD_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPDT(Modul, Space) sprintf(buffer, "/*M2_GEQPDT*/ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Ywt) = YD_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPDT(Modul, Space) sprintf(buffer, "/*M3_GEQPDT*/ \nvoid YgradEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YgradEQval : fonctions d'affectation de grad = val */
#define MACRO1_GEQV(Modul, Space)  sprintf(buffer, "/*M1_GEQV */ \nvoid YgradEQval_%s(double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YG_%s(Yws, Yw1) = val;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GEQV(Modul, Space)  sprintf(buffer, "/*M2_GEQV */ \nvoid YgradEQval_%s(double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Yw2) = val;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GEQV(Modul, Space)  sprintf(buffer, "/*M3_GEQV */ \nvoid YgradEQval_%s(double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3) = val;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_GEQVT(Modul, Space) sprintf(buffer, "/*M1_GEQVT*/ \nvoid YgradEQval_%s(int frompdt, int topdt, double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Ywt) = val;}}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GEQVT(Modul, Space) sprintf(buffer, "/*M2_GEQVT*/ \nvoid YgradEQval_%s(int frompdt, int topdt, double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Ywt) = val;}}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GEQVT(Modul, Space) sprintf(buffer, "/*M3_GEQVT*/ \nvoid YgradEQval_%s(int frompdt, int topdt, double val)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) = val;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul)

#endif
/*------------------------------------- FIN YO_VARINCR -------------------------------------*/


/*======================================== YO_TOOL ========================================*/
/* fonctions utilitaires   (nb: YO_GRADTEST=>YO_TOOL (see above)) */
#ifdef YO_TOOL

/*YdeltaEQPstate : fonctions d'affectation de delta = state * pfact pour les modules target */
#define MACRO1_DEQPS(Modul, Space)  sprintf(buffer, "/*M1_DEQPS */ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YD_%s(Yws, Yw1, Ywt) = YS_%s(Yws, Yw1) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_DEQPS(Modul, Space)  sprintf(buffer, "/*M2_DEQPS */ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YD_%s(Yws, Yw1, Yw2, Ywt) = YS_%s(Yws, Yw1, Yw2) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_DEQPS(Modul, Space)  sprintf(buffer, "/*M3_DEQPS */ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YS_%s(Yws, Yw1, Yw2, Yw3) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_DEQPST(Modul, Space) sprintf(buffer, "/*M1_DEQPST*/ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YD_%s(Yws, Yw1, Ywt) = YS_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_DEQPST(Modul, Space) sprintf(buffer, "/*M2_DEQPST*/ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Ywt) = YS_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_DEQPST(Modul, Space) sprintf(buffer, "/*M3_DEQPST*/ \nvoid YdeltaEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YdeltaEQPgrad : fonctions d'affectation de delta = grad * pfact pour les modules target */
#define MACRO1_DEQPG(Modul, Space)  sprintf(buffer, "/*M1_DEQPG */ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YD_%s(Yws, Yw1, Ywt) = YG_%s(Yws, Yw1) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_DEQPG(Modul, Space)  sprintf(buffer, "/*M2_DEQPG */ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YD_%s(Yws, Yw1, Yw2, Ywt) = YG_%s(Yws, Yw1, Yw2) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_DEQPG(Modul, Space)  sprintf(buffer, "/*M3_DEQPG */ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YG_%s(Yws, Yw1, Yw2, Yw3) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_DEQPGT(Modul, Space) sprintf(buffer, "/*M1_DEQPGT*/ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YD_%s(Yws, Yw1, Ywt) = YG_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_DEQPGT(Modul, Space) sprintf(buffer, "/*M2_DEQPGT*/ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Ywt) = YG_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_DEQPGT(Modul, Space) sprintf(buffer, "/*M3_DEQPGT*/ \nvoid YdeltaEQPgrad_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YstateEQPdelta : fonctions d'affectation de state = delta * pfact pour les modules target */
#define MACRO1_SEQPD(Modul, Space)  sprintf(buffer, "/*M1_SEQPD */ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YS_%s(Yws, Yw1) = YD_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQPD(Modul, Space)  sprintf(buffer, "/*M2_SEQPD */ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Yw2) = YD_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQPD(Modul, Space)  sprintf(buffer, "/*M3_SEQPD */ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_SEQPDT(Modul, Space) sprintf(buffer, "/*M1_SEQPDT*/ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Ywt) = YD_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQPDT(Modul, Space) sprintf(buffer, "/*M2_SEQPDT*/ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Ywt) = YD_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQPDT(Modul, Space) sprintf(buffer, "/*M3_SEQPDT*/ \nvoid YstateEQPdelta_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
/*YstateEQPgrad : fonctions d'affectation de state = grad * pfact (cf->v7)*/

/*YstateEQPOdelta : fonctions d'affectation de state = delta *  (pfact*(1+Ygetval(codop))) (Ygetval renvoie une valeur selon un code operation) */
#define MACRO1_SEQPOD(Modul, Space)  sprintf(buffer, "/*M1_SEQPOD */ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YS_%s(Yws, Yw1) = YD_%s(Yws, Yw1, Ywt) * (pfact*(1+Ygetval(codop)));}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQPOD(Modul, Space)  sprintf(buffer, "/*M2_SEQPOD */ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Yw2) = YD_%s(Yws, Yw1, Yw2, Ywt) * (pfact*(1+Ygetval(codop)));}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQPOD(Modul, Space)  sprintf(buffer, "/*M3_SEQPOD */ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * (pfact*(1+Ygetval(codop)));}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_SEQPODT(Modul, Space) sprintf(buffer, "/*M1_SEQPODT*/ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Ywt) = YD_%s(Yws, Yw1, Ywt) * (pfact*(1+Ygetval(codop)));}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQPODT(Modul, Space) sprintf(buffer, "/*M2_SEQPODT*/ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Ywt) = YD_%s(Yws, Yw1, Yw2, Ywt) * (pfact*(1+Ygetval(codop)));}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQPODT(Modul, Space) sprintf(buffer, "/*M3_SEQPODT*/ \nvoid YstateEQPOdelta_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) * (pfact*(1+Ygetval(codop)));}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YstateEQAPTdelta : fonctions d'affectation de state = delta + pfact*tab[Y3windice++] */
#define MACRO1_SEQAPTD(Modul, Space)  sprintf(buffer, "/*M1_SEQAPTD */ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YS_%s(Yws, Yw1) = YD_%s(Yws, Yw1, Ywt) + pfact * tab[Y3windice++];}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQAPTD(Modul, Space)  sprintf(buffer, "/*M2_SEQAPTD */ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Yw2) = YD_%s(Yws, Yw1, Yw2, Ywt) + pfact * tab[Y3windice++];}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQAPTD(Modul, Space)  sprintf(buffer, "/*M3_SEQAPTD */ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) + pfact * tab[Y3windice++];}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_SEQAPTDT(Modul, Space) sprintf(buffer, "/*M1_SEQAPTDT*/ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YS_%s(Yws, Yw1, Ywt) = YD_%s(Yws, Yw1, Ywt) + pfact * tab[Y3windice++];}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_SEQAPTDT(Modul, Space) sprintf(buffer, "/*M2_SEQAPTDT*/ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Ywt) = YD_%s(Yws, Yw1, Yw2, Ywt) + pfact * tab[Y3windice++];}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_SEQAPTDT(Modul, Space) sprintf(buffer, "/*M3_SEQAPTDT*/ \nvoid YstateEQAPTdelta_%s(int frompdt, int topdt, double pfact, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YD_%s(Yws, Yw1, Yw2, Yw3, Ywt) + pfact * tab[Y3windice++];}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YgradEQPstate : fonctions d'affectation de grad = state * pfact */
#define MACRO1_GEQPS(Modul, Space)  sprintf(buffer, "/*M1_GEQPS */ \nvoid YgradEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YG_%s(Yws, Yw1) = YS_%s(Yws, Yw1) * pfact;}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPS(Modul, Space)  sprintf(buffer, "/*M2_GEQPS */ \nvoid YgradEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Yw2) = YS_%s(Yws, Yw1, Yw2) * pfact;}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPS(Modul, Space)  sprintf(buffer, "/*M3_GEQPS */ \nvoid YgradEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3) = YS_%s(Yws, Yw1, Yw2, Yw3) * pfact;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_GEQPST(Modul, Space) sprintf(buffer, "/*M1_GEQPST*/ \nvoid YgradEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Ywt) = YS_%s(Yws, Yw1, Ywt) * pfact;}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPST(Modul, Space) sprintf(buffer, "/*M2_GEQPST*/ \nvoid YgradEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Ywt) = YS_%s(Yws, Yw1, Yw2, Ywt) * pfact;}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPST(Modul, Space) sprintf(buffer, "/*M3_GEQPST*/ \nvoid YgradEQPstate_%s(int frompdt, int topdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) * pfact;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)

/*YgradEQPOstate : fonctions d'affectation de grad = state * (pfact*(1+alea))  (Ygetval renvoie une valeur selon un code operation) */
#define MACRO1_GEQPOS(Modul, Space)  sprintf(buffer, "/*M1_GEQPOS */ \nvoid YgradEQPOstate_%s(double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YG_%s(Yws, Yw1) = YS_%s(Yws, Yw1) * (pfact*(1+Ygetval(codop)));}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPOS(Modul, Space)  sprintf(buffer, "/*M2_GEQPOS */ \nvoid YgradEQPOstate_%s(double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Yw2) = YS_%s(Yws, Yw1, Yw2) * (pfact*(1+Ygetval(codop)));}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPOS(Modul, Space)  sprintf(buffer, "/*M3_GEQPOS */ \nvoid YgradEQPOstate_%s(double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3) = YS_%s(Yws, Yw1, Yw2, Yw3) * (pfact*(1+Ygetval(codop)));}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_GEQPOST(Modul, Space) sprintf(buffer, "/*M1_GEQPOST*/ \nvoid YgradEQPOstate_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YG_%s(Yws, Yw1, Ywt) = YS_%s(Yws, Yw1, Ywt) * (pfact*(1+Ygetval(codop)));}}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_GEQPOST(Modul, Space) sprintf(buffer, "/*M2_GEQPOST*/ \nvoid YgradEQPOstate_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Ywt) = YS_%s(Yws, Yw1, Yw2, Ywt) * (pfact*(1+Ygetval(codop)));}}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_GEQPOST(Modul, Space) sprintf(buffer, "/*M3_GEQPOST*/ \nvoid YgradEQPOstate_%s(int frompdt, int topdt, double pfact, char *codop)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YG_%s(Yws, Yw1, Yw2, Yw3, Ywt) = YS_%s(Yws, Yw1, Yw2, Yw3, Ywt) * (pfact*(1+Ygetval(codop)));}}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)


/*YgradPEPstate : fonctions d'affectation de grad += state * pfact (cf->v7) */
/*YstatePEPgrad : fonctions d'affectation de state += grad * pfact (cf->v7) */

/*YwishEQPstate : fonctions d'affectation de wish = state * pfact */
#define MACRO1_WEQPS(Modul, Space)  sprintf(buffer, "/*M1_WEQPS */ \nvoid YwishEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t YW_%s(Yws, Yw1) = YS_%s(Yws, Yw1) * pfact;}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_WEQPS(Modul, Space)  sprintf(buffer, "/*M2_WEQPS */ \nvoid YwishEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YW_%s(Yws, Yw1, Yw2) = YS_%s(Yws, Yw1, Yw2) * pfact;}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_WEQPS(Modul, Space)  sprintf(buffer, "/*M3_WEQPS */ \nvoid YwishEQPstate_%s(double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YW_%s(Yws, Yw1, Yw2, Yw3) = YS_%s(Yws, Yw1, Yw2, Yw3) * pfact;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)
#define MACRO1_WEQPST(Modul, Space) sprintf(buffer, "/*M1_WEQPST*/ \nvoid YwishEQPstate_%s(int pdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t YW_%s(Yws, Yw1) = YS_%s(Yws, Yw1, pdt) * pfact;}}\n}", Modul, Space, Modul, Modul, Modul)
#define MACRO2_WEQPST(Modul, Space) sprintf(buffer, "/*M2_WEQPST*/ \nvoid YwishEQPstate_%s(int pdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t YW_%s(Yws, Yw1, Yw2) = YS_%s(Yws, Yw1, Yw2, pdt) * pfact;}}}\n}", Modul, Space, Space, Modul, Modul, Modul)
#define MACRO3_WEQPST(Modul, Space) sprintf(buffer, "/*M3_WEQPST*/ \nvoid YwishEQPstate_%s(int pdt, double pfact)\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t YW_%s(Yws, Yw1, Yw2, Yw3) = YS_%s(Yws, Yw1, Yw2, Yw3, pdt) * pfact;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, Modul)


/*YstateTOtab : fonctions de transfert des states dans un tableau */
#define MACRO1_STOTB(Modul, Space)  sprintf(buffer, "/*M1_STOTB   */ \nvoid YstateTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1); ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_STOTB(Modul, Space)  sprintf(buffer, "/*M2_STOTB   */ \nvoid YstateTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1, Yw2); ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_STOTB(Modul, Space)  sprintf(buffer, "/*M3_STOTB   */ \nvoid YstateTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1, Yw2, Yw3); ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_STOTBT(Modul, Space) sprintf(buffer, "/*M1_STOTBT  */ \nvoid YstateTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1, Ywt); ++Y3windice;}}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_STOTBT(Modul, Space) sprintf(buffer, "/*M2_STOTBT  */ \nvoid YstateTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1, Yw2, Ywt); ++Y3windice;}}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_STOTBT(Modul, Space) sprintf(buffer, "/*M3_STOTBT  */ \nvoid YstateTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t tab[Y3windice] = YS_%s(Yws, Yw1, Yw2, Yw3, Ywt); ++Y3windice;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul)

/*YgradTOtab : fonctions de transfert des gradients dans un tableau */
#define MACRO1_GTOTB(Modul, Space)  sprintf(buffer, "/*M1_GTOTB   */ \nvoid YgradTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1); ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GTOTB(Modul, Space)  sprintf(buffer, "/*M2_GTOTB   */ \nvoid YgradTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1, Yw2); ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GTOTB(Modul, Space)  sprintf(buffer, "/*M3_GTOTB   */ \nvoid YgradTOtab_%s(YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1, Yw2, Yw3); ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_GTOTBT(Modul, Space) sprintf(buffer, "/*M1_GTOTBT  */ \nvoid YgradTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Ywt=frompdt; Ywt<topdt;  ++Ywt){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1, Ywt); ++Y3windice;}}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GTOTBT(Modul, Space) sprintf(buffer, "/*M2_GTOTBT  */ \nvoid YgradTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1, Yw2, Ywt); ++Y3windice;}}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GTOTBT(Modul, Space) sprintf(buffer, "/*M3_GTOTBT  */ \nvoid YgradTOtab_%s(int frompdt, int topdt, YREAL tab[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3){\n\t\t\t\t for (int Ywt=frompdt; Ywt<topdt; ++Ywt){\n\t\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t\t tab[Y3windice] = YG_%s(Yws, Yw1, Yw2, Yw3, Ywt); ++Y3windice;}}}}}\n}", Modul, Space, Space, Space, Modul, Modul)

/*YgradCTOtab : fonctions de cumul des gradients dans un tableau (pour basic intern loop) : cf ci-dessus 'sans option' */
/*YtabTOgrad : fonctions de transfert d'un tableau vers les gradients (deplace ci-dessus : cf 'sans option ' */

#endif
/*--------------------------------------- FIN YO_TOOL --------------------------------------*/


/*======================================== YO_M1QN3 ========================================*/
/*Fonctions necessaires pour m1qn3 (sauf a revoir les structures de donnees (x et g) pour qu'elles soient communes (en fait elles le seraient)
//les modules qui sont concernes sont les targets. mais un target peut-il etre cout et/ou tempo (plutot oui)
//Y3getstate : affectation du tableau des etats m1qn3->Yao */

#ifdef YO_M1QN3
#define MACRO1_GSTA(Modul, Space) sprintf(buffer, "/*M1_GSTA*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y%s->%s[Yw1][Yws] = x[Y3windice]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YSTATE)
#define MACRO2_GSTA(Modul, Space) sprintf(buffer, "/*M2_GSTA*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y%s->%s[Yw1][Yw2][Yws] = x[Y3windice]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_GSTA(Modul, Space) sprintf(buffer, "/*M3_GSTA*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y%s->%s[Yw1][Yw2][Yw3][Yws] = x[Y3windice]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_GSTAT(Modul, Space) sprintf(buffer, "/*M1_GSTAT*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y%s->%s[Ywt][Yw1][Yws] = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO2_GSTAT(Modul, Space) sprintf(buffer, "/*M2_GSTAT*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y%s->%s[Ywt][Yw1][Yw2][Yws] = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO3_GSTAT(Modul, Space) sprintf(buffer, "/*M3_GSTAT*/ \nvoid Y3getstate_%s(float x[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws] = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YSTATE)

/*Y3valstate : affectation du tableau des etats Yao->m1qn3 */
// done.
#define MACRO1_VSTA(Modul, Space) sprintf(buffer, "/*M1_VSTA*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y3x[Y3windice] = Y%s->%s[Yw1][Yws]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YSTATE)
#define MACRO2_VSTA(Modul, Space) sprintf(buffer, "/*M2_VSTA*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y3x[Y3windice] = Y%s->%s[Yw1][Yw2][Yws]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_VSTA(Modul, Space) sprintf(buffer, "/*M3_VSTA*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y3x[Y3windice] = Y%s->%s[Yw1][Yw2][Yw3][Yws]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_VSTAT(Modul, Space) sprintf(buffer, "/*M1_VSTAT*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Y%s->%s[Ywt][Yw1][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO2_VSTAT(Modul, Space) sprintf(buffer, "/*M2_VSTAT*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO3_VSTAT(Modul, Space) sprintf(buffer, "/*M3_VSTAT*/ \nvoid Y3valstate_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YSTATE)

/*Y3valgrad : affectation du tableau des gradients Yao->m1qn3 */
// done
#define MACRO1_VGRAD(Modul, Space) sprintf(buffer, "/*M1_VGRAD*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t g[Y3windice] = Y%s->%s[Yw1][Yws]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YGRAD)
#define MACRO2_VGRAD(Modul, Space) sprintf(buffer, "/*M2_VGRAD*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t g[Y3windice] = Y%s->%s[Yw1][Yw2][Yws]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YGRAD)
#define MACRO3_VGRAD(Modul, Space) sprintf(buffer, "/*M3_VGRAD*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t g[Y3windice] = Y%s->%s[Yw1][Yw2][Yw3][Yws]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YGRAD)
#define MACRO1_VGRADT(Modul, Space) sprintf(buffer, "/*M1_VGRADT*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { g[Y3windice] = Y%s->%s[Ywt][Yw1][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YGRAD)
#define MACRO2_VGRADT(Modul, Space) sprintf(buffer, "/*M2_VGRADT*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { g[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YGRAD)
#define MACRO3_VGRADT(Modul, Space) sprintf(buffer, "/*M3_VGRADT*/ \nvoid Y3valgrad_%s(float g[])\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { g[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YGRAD)

/*Y3valxinf : affectation du tableau des bornes inf pour m2qn1: etats Yao->m2qn1 (Y3xinf) */
// done
#define MACRO1_VXINF(Modul, Space) sprintf(buffer, "/*M1_VXINF*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y3xinf[Y3windice] = Y%s->%s[Yw1][Yws]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YSTATE)
#define MACRO2_VXINF(Modul, Space) sprintf(buffer, "/*M2_VXINF*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y3xinf[Y3windice] = Y%s->%s[Yw1][Yw2][Yws]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_VXINF(Modul, Space) sprintf(buffer, "/*M3_VXINF*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y3xinf[Y3windice] = Y%s->%s[Yw1][Yw2][Yw3][Yws]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_VXINFT(Modul, Space) sprintf(buffer, "/*M1_VXINFT*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xinf[Y3windice] = Y%s->%s[Ywt][Yw1][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO2_VXINFT(Modul, Space) sprintf(buffer, "/*M2_VXINFT*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xinf[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO3_VXINFT(Modul, Space) sprintf(buffer, "/*M3_VXINFT*/ \nvoid Y3valxinf_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xinf[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YSTATE)

/*Y3valxsup : affectation du tableau des bornes sup pour m2qn1: etats Yao->m2qn1 (Y3xsup) */
// done
#define MACRO1_VXSUP(Modul, Space) sprintf(buffer, "/*M1_VXSUP*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y3xsup[Y3windice] = Y%s->%s[Yw1][Yws]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YSTATE)
#define MACRO2_VXSUP(Modul, Space) sprintf(buffer, "/*M2_VXSUP*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y3xsup[Y3windice] = Y%s->%s[Yw1][Yw2][Yws]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_VXSUP(Modul, Space) sprintf(buffer, "/*M3_VXSUP*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y3xsup[Y3windice] = Y%s->%s[Yw1][Yw2][Yw3][Yws]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_VXSUPT(Modul, Space) sprintf(buffer, "/*M1_VXSUPT*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xsup[Y3windice] = Y%s->%s[Ywt][Yw1][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO2_VXSUPT(Modul, Space) sprintf(buffer, "/*M2_VXSUPT*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xsup[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO3_VXSUPT(Modul, Space) sprintf(buffer, "/*M3_VXSUPT*/ \nvoid Y3valxsup_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3xsup[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YSTATE)

/*Y3valdxmin : affectation du tableau des bornes sup pour m2qn1: etats Yao->m2qn1 (Y3dxmin) */
// done
#define MACRO1_VDXMI(Modul, Space) sprintf(buffer, "/*M1_VDXMI*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y3dxmin[Y3windice] = Y%s->%s[Yw1][Yws]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul, YSTATE)
#define MACRO2_VDXMI(Modul, Space) sprintf(buffer, "/*M2_VDXMI*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y3dxmin[Y3windice] = Y%s->%s[Yw1][Yw2][Yws]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul, YSTATE)
#define MACRO3_VDXMI(Modul, Space) sprintf(buffer, "/*M3_VDXMI*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1){\n\t\t for (int Yw2=0; Yw2<YA2_%s;  ++Yw2){\n\t\t\t for (int Yw3=0; Yw3<YA3_%s;  ++Yw3){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y3dxmin[Y3windice] = Y%s->%s[Yw1][Yw2][Yw3][Yws]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul, YSTATE)
#define MACRO1_VDXMIT(Modul, Space) sprintf(buffer, "/*M1_VDXMIT*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3dxmin[Y3windice] = Y%s->%s[Ywt][Yw1][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO2_VDXMIT(Modul, Space) sprintf(buffer, "/*M2_VDXMIT*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3dxmin[Y3windice] = Y%s(Yw1, Yw2)->%s[Ywt][Yw1][Yw2][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul, YSTATE)
#define MACRO3_VDXMIT(Modul, Space) sprintf(buffer, "/*M3_VDXMIT*/ \nvoid Y3valdxmin_%s()\n{\n\t for (int Yw1=0; Yw1<YA1_%s; ++Yw1)\n\t for (int Yw2=0; Yw2<YA2_%s; ++Yw2)\n\t for (int Yw3=0; Yw3<YA3_%s; ++Yw3)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3dxmin[Y3windice] = Y%s->%s[Ywt][Yw1][Yw2][Yw3][Yws]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul, YSTATE)

#ifdef YO_VARINCR
/*Y3valdelta : affectation du tableau des delta Yao->m1qn3 */
#define MACRO1_VDEL(Modul, Space) sprintf(buffer, "/*M1_VDEL*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, 0); ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_VDEL(Modul, Space) sprintf(buffer, "/*M2_VDEL*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Ywj=0; Ywj<YA2_%s;  ++Ywj){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, Ywj, 0); ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_VDEL(Modul, Space) sprintf(buffer, "/*M3_VDEL*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Ywj=0; Ywj<YA2_%s;  ++Ywj){\n\t\t\t for (int Ywk=0; Ywk<YA3_%s;  ++Ywk){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, Ywj, Ywk, 0); ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_VDELT(Modul, Space) sprintf(buffer, "/*M1_VDELT*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, Ywt); ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul)
#define MACRO2_VDELT(Modul, Space) sprintf(buffer, "/*M2_VDELT*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Ywj=0; Ywj<YA2_%s; ++Ywj)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, Ywj, Ywt); ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO3_VDELT(Modul, Space) sprintf(buffer, "/*M3_VDELT*/ \nvoid Y3valdelta_%s()\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Ywj=0; Ywj<YA2_%s; ++Ywj)\n\t for (int Ywk=0; Ywk<YA3_%s; ++Ywk)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Y3x[Y3windice] = Ydelta_%s(Yws, Ywi, Ywj, Ywk, Ywt); ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul)

/*Y3getdelta : affectation du tableau des delta m1qn3->Yao */
#define MACRO1_GDEL(Modul, Space) sprintf(buffer, "/*M1_GDEL*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t Ydelta_%s(Yws, Ywi, 0) = x[Y3windice]; ++Y3windice;}}\n}", Modul, Space, Modul, Modul)
#define MACRO2_GDEL(Modul, Space) sprintf(buffer, "/*M2_GDEL*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Ywj=0; Ywj<YA2_%s;  ++Ywj){\n\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t Ydelta_%s(Yws, Ywi, Ywj, 0) = x[Y3windice]; ++Y3windice;}}}\n}", Modul, Space, Space, Modul, Modul)
#define MACRO3_GDEL(Modul, Space) sprintf(buffer, "/*M3_GDEL*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi){\n\t\t for (int Ywj=0; Ywj<YA2_%s;  ++Ywj){\n\t\t\t for (int Ywk=0; Ywk<YA3_%s;  ++Ywk){\n\t\t\t\t for (int Yws=0; Yws<YNBS_%s; ++Yws){\n\t\t\t\t\t Ydelta_%s(Yws, Ywi, Ywj, Ywk, 0) = x[Y3windice]; ++Y3windice;}}}}\n}", Modul, Space, Space, Space, Modul, Modul)
#define MACRO1_GDELT(Modul, Space) sprintf(buffer, "/*M1_GDELT*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Ydelta_%s(Yws, Ywi, Ywt) = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Modul, Modul, Modul, Modul)
#define MACRO2_GDELT(Modul, Space) sprintf(buffer, "/*M2_GDELT*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Ywj=0; Ywj<YA2_%s; ++Ywj)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Ydelta_%s(Yws, Ywi, Ywj, Ywt) = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Modul, Modul, Modul, Modul)
#define MACRO3_GDELT(Modul, Space) sprintf(buffer, "/*M3_GDELT*/ \nvoid Y3getdelta_%s(float x[])\n{\n\t for (int Ywi=0; Ywi<YA1_%s; ++Ywi)\n\t for (int Ywj=0; Ywj<YA2_%s; ++Ywj)\n\t for (int Ywk=0; Ywk<YA3_%s; ++Ywk)\n\t for (int Yws=0; Yws<YNBS_%s; ++Yws)\n\t for (int Ywt=YDEBTARGET_%s; Ywt<YENDTARGET_%s; ++Ywt)\n\t { Ydelta_%s(Yws, Ywi, Ywj, Ywk, Ywt) = x[Y3windice]; ++Y3windice;\n\t }\n}", Modul, Space, Space, Space, Modul, Modul, Modul, Modul)
#endif

#endif
/*-------------------------------------- FIN YO_M1QN3 --------------------------------------*/


/*======================================== YO_GRADTEST ======================================*/
/*Ygradtest : fonction de test du calcul des derivees */
#ifdef YO_GRADTEST
//#define MACRO00_GTEST(Modul) sprintf(buffer, "/*M00_GTEST */\nint Ytestdf_%s (int modop, int KeKo, float pdx, float ptol, int ynbi, int ynbs, char *nmmod, YREAL yinput[], YREAL ystate[], %s *Modul)\n{\t int wk, wj, nbKo=0; YREAL wSj[YMAX_NBS],wdxk,wJjk,wyj,wDjk,wdifjk;\n\t YREAL svx; int resKo; char okko[2][3]={\"ok\", \"KO\"}; \n\t if (!Ydftestijkt(Yid_%s)) return(0); \n\t memset (Yjac, 0, YMAX_JAC_NBI*YMAX_JAC_NBS*sizeof(YREAL));\n", Modul, Modul, Modul)
 //#define MACRO01_GTESTT(Parm) sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[YTemps*ynbs + wj];\n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[YTemps*ynbs + wj];\n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
//#define MACRO01_GTEST(Parm)  sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];              \n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];              \n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')

#define MACRO00_GTEST(Modul) sprintf(buffer, "/*M00_GTEST */\nint Ytestdf_%s (int modop, int KeKo, float pdx, float ptol, int ynbi, int ynbs, char *nmmod, YREAL yinput[], YREAL ystate[], %s *Modul)\n{\t int wk, wj, nbKo=0; YREAL wSj[YMAX_NBS],wdxk,wJjk,wyj,wDjk,wdifjk;\n\t YREAL svx; int resKo; char okko[2][3]={\"ok\", \"KO\"}; \n\t if (!Ydftestijkt(Yid_%s)) return(0); \n\t memset (Yjac, 0, YMAX_JAC_NBI*YMAX_JAC_NBS*sizeof(YREAL));\n", Modul, Modul, Modul)
//#define MACRO00_GTEST(Modul) sprintf(buffer, "/*M00_GTEST */\nint Ytestdf_%s (int modop, int KeKo, float pdx, float ptol, int ynbi, int ynbs, char *nmmod, YREAL yinput[], YREAL ystate[], %s *Modul)\n{\t int wk, wj, nbKo=0; YREAL wSj[YMAX_NBS],wdxk,wJjk,wyj,wDjk,wdifjk;\n\t YREAL svx; int resKo; char okko[2][3]={\"ok\", \"KO\"}; \n\t if (!Ydftestijkt(Yid_%s)) return(0); \n\t memset (Yjac, 0, YMAX_JAC_NBI*YMAX_JAC_NBS*sizeof(YREAL));\n", Modul, Modul, Modul)
//#define MACRO00_GTEST(Modul) sprintf(buffer, "/*M00_GTEST */\nint Ytestdf_%s (int modop, int KeKo, float pdx, float ptol, int ynbi, int ynbs, char *nmmod, YREAL yinput[], YREAL ystate[][], %s *Modul)\n{\t int wk, wj, nbKo=0; YREAL wSj[YMAX_NBS],wdxk,wJjk,wyj,wDjk,wdifjk;\n\t YREAL svx; int resKo; char okko[2][3]={\"ok\", \"KO\"}; \n\t if (!Ydftestijkt(Yid_%s)) return(0); \n\t memset (Yjac, 0, YMAX_JAC_NBI*YMAX_JAC_NBS*sizeof(YREAL));\n", Modul, Modul, Modul)
//#define MACRO00_GTEST(Modul) sprintf(buffer, "/*M00_GTEST */\nint Ytestdf_%s (int modop, int KeKo, float pdx, float ptol, int ynbi, int ynbs, char *nmmod, YREAL yinput[], YREAL ystate[][][], %s *Modul)\n{\t int wk, wj, nbKo=0; YREAL wSj[YMAX_NBS],wdxk,wJjk,wyj,wDjk,wdifjk;\n\t YREAL svx; int resKo; char okko[2][3]={\"ok\", \"KO\"}; \n\t if (!Ydftestijkt(Yid_%s)) return(0); \n\t memset (Yjac, 0, YMAX_JAC_NBI*YMAX_JAC_NBS*sizeof(YREAL));\n", Modul, Modul, Modul)

 #define MACRO11_GTESTT(Parm) sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];\n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];\n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
#define MACRO11_GTEST(Parm)  sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];              \n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];              \n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
 #define MACRO21_GTESTT(Parm) sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];\n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];\n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
#define MACRO21_GTEST(Parm)  sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];              \n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];              \n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
 #define MACRO31_GTESTT(Parm) sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];\n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];\n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
#define MACRO31_GTEST(Parm)  sprintf(buffer, "/*M01_GTESTT*/\n\t Modul->forward(%s);\n\t for (wj=0; wj<ynbs; ++wj) wSj[wj] = ystate[wj];              \n\t Modul->backward(%s);\n\t for (wk=0; wk<ynbi; ++wk)\n\t {\t svx=yinput[wk];\n\t\t\t wdxk = svx*pdx;\n\t\t\t if (modop||svx==0.0) wdxk = pdx; \n\t\t\t yinput[wk] += wdxk;\n\t\t\t Modul->forward(%s);\n\t\t\t for (wj=0; wj<ynbs; ++wj)\n\t\t\t { wJjk = Yjac[wj][wk];\n\t\t\t\t wyj = ystate[wj];              \n\t\t\t\t wDjk = (wyj - wSj[wj]) / wdxk;\n\t\t\t\t wdifjk = fabs(wDjk - wJjk);\n\t\t\t\t resKo=(wdifjk/fabs(wJjk))>ptol;  nbKo+=resKo;\n\t\t\t\t if(resKo || !KeKo) \n\t\t\t\t printf(\" %c7s | J[%c2i,%c2i] |%c 12.6e |%c 12.6e |%c 12.6e :  %cs |%c 12.6e | (%ci %ci %ci %ci)\\n\",\n\t\t\t\t\t\t\t\t nmmod, wj+1, wk+1, wJjk, wDjk, wdifjk, okko[resKo], svx, Yi+1, Yj+1, Yk+1, Yt+1);\n\t\t\t\t} \n\t\t\t  yinput[wk] = svx;\n\t }\n\t return(nbKo);\n}", Parm, Parm, Parm, '%','%','%','%','%','%','%','%','%','%','%','%')
#endif
/*------------------------------------- FIN YO_GRADTEST ------------------------------------*/

/* SOME DEFINES //////////////////////////////////////////////////////////// */
#define BUFSIZE		      4096  //<! Max dimension of the buffer.
#define NB_MAX_MACRO	      50    //<! Max number of macros that can be generated. This is a way to limit the explosion of the files generated.
#define LG_MAX_STRING	      512   //<! Max dimension of an array, various usage (LG=longeur).
//#define NB_MAX_CONNECT	      10000 //<! Max number of connection for the graphe.
#define NB_MAX_CONNECT	      100000 //<! Max number of connection for the graphe.
#define NB_MAX_ORDER_TOKENS   800   //<! Max number of token for the order directive.

namespace yao{

//! Translator class, handles both project description analysis and code generation.
class Translator: public BaseParser{
  friend class TemplateAnalyzer;

 public:

  Translator(Tokenizer& aTokenizer); //!< Constructor.

  //! Analyzes project description then generates (declaration and implementation) code.
  void generateCode();

  void setNewStructure(bool aBoolean);
  void setPathTemplate(const std::string& aPath);
  void setPathOutput(const std::string& aPath);
  void setPathInclude(const std::string& aPath);
  void setActivateNamespace(bool aBoolean);

 private:
  //! Templates path. Initialized to "$YAODIR/yao/templates" by default.
  FilePath thePathTemplate;

  //! Output path. Initialized to "./" by default.
  FilePath thePathOutput;

  //! Include source path. Not used for now. Initialized to "$YAODIR/yao/include" by default.
  FilePath thePathInclude;

  //! Does the user wants to use the new Yao File structure ?
  bool isNewStructure;

  //! Does the user wants a namespace project ?
  bool isNamespace;

  //! String which will contain the loops generated source. For now, the key is the trajectory name.
  map<string,string> forwardLoopSrc, backwardLoopSrc, linwardLoopSrc, dfwardLoopSrc;
  //!!temp
  void testTemplateAnalyzer();

  void templateGenerator();


  //ofstream declaration, externfile, implementation;

  void createYaoFile(bool isSource, ofstream & file, string aFilename);


  void generateCode(std::ostream& anOutput, const Table<Operator>& anOperatorTable);
  void generateCode(ostream& aDeclaration, ostream& anExternal, Context& theContext);
  void generateCode(ostream& aDeclaration, Table<Trajectory>& aTrajectoryTable);
  void generateCode(ostream& aDeclaration, ostream& anImplementation, const Table<Neuron>& aNeuronTable);
  void generateCode(ostream& aDeclaration, ostream& anImplementation, Table<Modul>& aModuleTable);
  void generateCode(Table<Connection>& aConnectionTable);
  void generateCode(ostream& anImplementation, Table<Order>& anOrderTable);
  void generateCode(ostream& anImplementation, Table<Function>& aFunctionTable);
  void generateCode(ostream& aDeclaration, ostream& anImplementation);

  void createYaoModules(ostream& aDeclaration);
  void norkmod(ostream& anImplementation, Table<Modul>::iterator mod);
  void dfwardAllGenerator(ostream& anImplementation);
  void stockCheckConnection(Table<Connection> & allConnectionTable, Connection newCon);
  // From the next method all the methods are for the order directive.
  void orderGenerator(ostream& anImplementation, string wardKind, Order ord);
  void orderGeneratorMulti(ostream& anImplementation, string wardKind, Order ord);
  void flagAxe(bool axeTable[3], int axe, int & dimaxe);
  void unflagAxe(bool axeTable[3], Order ord, vector<string>::iterator tokenIndex, int & dimAxe);
  void searchAxes(bool axeTable[3], char *a, char *b);
  void dfwardGeneratorOrderModule(ostream& anImplementation, Order ord, Modul mod, int dimAxe, char a, char b);
  void feedward(ostream& anImplementation, Order ord, string inModule, char ibad, int firstAxis, bool parallel);
  void dfwardGridGenerator(ostream& anImplementation, Order ord);
//  void dfwardOperaGridGenerator(ostream& anImplementation);
  void dfwardGeneratorModule(ostream& anImplementation, Order ord, Modul mod, int dimAxe, char a, char b);
  void wardOperatorGenerator(ostream &anImplementation);
  bool parallelVerification(vector<string>::iterator token, Order ord, string wardKind);
};

} // namespace yao

#endif // !YAO_TRANSLATOR_HPP_INCLUDED
